#!/usr/bin/ruby

# -------------------------------------------------------------------------
# Converts github-flavored markdown to HTML with syntax highlighting of
# embedded code.
# -------------------------------------------------------------------------

# https://github.com/vmg/redcarpet
# sudo gem install redcarpet
require "redcarpet"

# sudo easy_install pygments
# sudo gem install pygments.rb
require "pygments.rb"

Class_prefix = 'ph-'
# Pygments styles: ['manni', 'igor', 'xcode', 'vim', 'autumn', 'vs', 'rrt', 'native', 'perldoc', 'borland', 'tango', 'emacs', 'friendly', 'monokai', 'paraiso-dark', 'colorful', 'murphy', 'bw', 'pastie', 'paraiso-light', 'trac', 'default', 'fruity']
Highlight_style = 'default'
Code_block_prefix = "<div class=\"highlight\"><pre>"
Code_block_suffix = "</pre></div>"


# derived from https://github.com/alampros/Docter/blob/master/bin/github-flavored-markdown.rb
class HTMLwithPygments < Redcarpet::Render::HTML
    def block_code(code, language)
        Pygments.highlight(code, :lexer => language, :options => {:encoding => 'utf-8', :cssclass => 'code', :classprefix => Class_prefix, :style => Highlight_style})
    end
    def codespan(code)
        # Guessing swift for now
        block = Pygments.highlight(code, :lexer => 'swift', :options => {:encoding => 'utf-8', :classprefix => Class_prefix, :style => Highlight_style})
        # Now strip wrapping:
        block.slice!(Code_block_prefix)
        block.slice!(Code_block_suffix)
        block.strip!
        "<code>#{block}</code>"
    end
    def normal_text(text)
        # Don't let Redcarpet translate " to &quot; or SmartyPants won't work
        "#{text}"
    end
end

def translate(markdown_text)
    renderer = HTMLwithPygments.new() #(with_toc_data: true)
    markdown = Redcarpet::Markdown.new(renderer, autolink: true, disable_indented_code_blocks: true, fenced_code_blocks: true, footnotes: true, strikethrough: true, superscript: true, tables: true)
    markdown.render(markdown_text)  
end

# Generate css
# css = Pygments.css(:style => Highlight_style, :classprefix => Class_prefix)
# css.sub!(/.hll/, ".code")
# puts css
# 
# 

def usage()
  puts "gf_markdown.rb markdownFile"
  puts "gf_markdown.rb -"
end

def validateAndRead(fileName)
  unless File.file?(fileName)
    puts "#{fileName} is not a file"
    exit(-1)
  end
  File.open(fileName, mode="r") { |file| file.read() } 
end

if ARGV.count != 1
  usage()
  exit(-1)
end

if ARGV[0] == "-"
  markdown_text = STDIN.read()
else 
  markdown_text = validateAndRead(ARGV[0])
end

result = translate(markdown_text)

puts result

